﻿Type=Class
Version=6.5
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
'version: 1.00
#Event: AccessTokenAvailable (Success As Boolean, Token As String)
#ExcludeFromLibrary: True
Sub Class_Globals
	#if B4A
	Private LastIntent As Intent
	#end if
	Private mTarget As Object
	Private mEventName As String
	Private mClientId As String
	Private mScope As String
	Type TokenInformation (AccessToken As String, RefreshToken As String, AccessExpiry As Long, Valid As Boolean)
	Private ti As TokenInformation
	Private Const TokenFile As String = "oauth2token.dat"
	Private TokenFolder As String
	Private packageName As String
End Sub

Public Sub Initialize (Target As Object, EventName As String, ClientId As String, Scope As String)
	mTarget = Target
	mEventName = EventName
	mClientId = ClientId
	mScope = Scope
	#if B4A
		packageName = Application.PackageName
		TokenFolder = File.DirInternal
	#Else
		TokenFolder = File.DirLibrary
		packageName = GetPackageName
	#End If
	If File.Exists(TokenFolder, TokenFile) Then
		Dim raf As RandomAccessFile
		raf.Initialize(TokenFolder, TokenFile, True)
		ti = raf.ReadB4XObject(raf.CurrentPosition)
		raf.Close
	End If
End Sub

Private Sub SaveToken
	Dim raf As RandomAccessFile
	raf.Initialize(TokenFolder, TokenFile, False)
	raf.WriteB4XObject(ti, raf.CurrentPosition)
	raf.Close
End Sub

Public Sub ResetToken
	ti.Valid = False
	SaveToken
End Sub

Public Sub GetAccessToken
	If ti.Valid = False Then
		Authenticate
	Else if ti.AccessExpiry > DateTime.Now Then
		GetTokenFromRefresh
	Else
		RaiseEvent(True)
	End If
End Sub

Private Sub Authenticate
	Dim link As String = BuildLink("https://accounts.google.com/o/oauth2/v2/auth", _
		 CreateMap("client_id": mClientId, _
		"redirect_uri": packageName & ":/oath", _
		"response_type": "code", "scope": mScope))
#if B4A
	Dim pi As PhoneIntents
	StartActivity(pi.OpenBrowser(link))
#else
	Main.App.OpenURL(link)
#end if
End Sub

#if B4A
Public Sub CallFromResume(Intent As Intent)
	If IsNewOAuth2Intent(Intent) Then
		LastIntent = Intent
		ParseBrowserUrl(Intent.GetData)
	End If
End Sub

Private Sub IsNewOAuth2Intent(Intent As Intent) As Boolean
	Return Intent.IsInitialized And Intent <> LastIntent And Intent.Action = Intent.ACTION_VIEW And _
		Intent.GetData <> Null And Intent.GetData.StartsWith(Application.PackageName)
End Sub
#else
Public Sub CallFromOpenUrl (url As String)
	If url.StartsWith(packageName & ":/oath") Then
		ParseBrowserUrl(url)
	End If
End Sub

Private Sub GetPackageName As String
	Dim no As NativeObject
	no = no.Initialize("NSBundle").RunMethod("mainBundle", Null)
	Dim name As Object = no.RunMethod("objectForInfoDictionaryKey:", Array("CFBundleIdentifier"))
	Return name
End Sub

#end if

Private Sub ParseBrowserUrl(Response As String)
	Dim m As Matcher = Regex.Matcher("code=([^&]+)", Response)
	If m.Find Then
		Dim code As String = m.Group(1)
		GetTokenFromAuthorizationCode(code)
	Else
		Log("Error parsing server response: " & Response)
		ResetToken
		RaiseEvent(False)
	End If
End Sub

Private Sub RaiseEvent(Success As Boolean)
	CallSubDelayed3(mTarget, mEventName & "_AccessTokenAvailable", Success, ti.AccessToken)
End Sub


Private Sub GetTokenFromAuthorizationCode (Code As String)
	Log("Getting access token from authorization code...")
	Dim j As HttpJob
	j.Initialize("", Me)
	j.PostString("https://www.googleapis.com/oauth2/v4/token", _
		$"code=${Code}&client_id=${mClientId}&grant_type=authorization_code&redirect_uri=${packageName & ":/oath"}"$)
'	Wait For (j) JobDone(j As HttpJob)
'	If j.Success Then
'		TokenInformationFromResponse(j.GetString)
'	Else
'		ResetToken
'		RaiseEvent(False)
'	End If
'	j.Release
End Sub

Private Sub GetTokenFromRefresh
	Log("Getting access token from refresh token...")
	Dim j As HttpJob
	j.Initialize("", Me)
	j.PostString("https://www.googleapis.com/oauth2/v4/token", _
		$"refresh_token=${ti.RefreshToken}&client_id=${mClientId}&grant_type=refresh_token&redirect_uri=${packageName & ":/oath"}"$)
'	Wait For (j) JobDone(j As HttpJob)
'	If j.Success Then
'		TokenInformationFromResponse(j.GetString)
'	Else
'		RaiseEvent(False)
'	End If
'	j.Release
End Sub

Private Sub TokenInformationFromResponse (s As String)
	Dim jp As JSONParser
	jp.Initialize(s)
	Dim m As Map = jp.NextObject
	ti.AccessExpiry = DateTime.Now + m.Get("expires_in")
	ti.AccessToken = m.Get("access_token")
	If m.ContainsKey("refresh_token") Then ti.RefreshToken = m.Get("refresh_token")
	ti.Valid = True
	SaveToken
	RaiseEvent(True)
End Sub



Private Sub BuildLink(Url As String, Params As Map) As String
	Dim su As StringUtils
	Dim sb As StringBuilder
	sb.Initialize
	sb.Append(Url)
	If Params.Size > 0 Then
		sb.Append("?")
		For Each k As String In Params.Keys
			sb.Append(su.EncodeUrl(k, "utf8")).Append("=").Append(su.EncodeUrl(Params.Get(k), "utf8"))
			sb.Append("&")
		Next
		sb.Remove(sb.Length - 1, sb.Length)
	End If
	Return sb.ToString
End Sub


Sub  JobDone(j As HttpJob)
	If j.Success Then
		TokenInformationFromResponse(j.GetString)
	Else
		ResetToken
		RaiseEvent(False)
	End If
	j.Release
End Sub 
